// remapt-1.c
// The tool to enhance t-1 frames by remapping the palette
// 16.09.2015
// 
// Copyright (C) 2015  Balthasar Szczepański
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
// 
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

// Requires Dev Image Library (libdevil) (http://openil.sourceforge.net/)
// on Pentium III libdevil must be recompiled with
// --disable-ssl2 --disable-ssl3
// (https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=572954)

//#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "IL/il.h"

int mustard(const char *t,int m,int e);
int main(int argc, char *argv[]);
unsigned char b642d(char b64);

ILuint inpix;
unsigned char q =0;

int main(int argc, char *argv[])
{
	ILubyte *pal;
	unsigned short i;
	
	if (argc<4)
		return mustard("remapt-1 inpix outpix palette [q]",0,1);
  else if(argc > 4)
		q=1;
	
	if(strlen(argv[3])<340)
		return mustard("Palette too short.",0,1);
	
	ilInit();
	
	ilEnable(IL_ORIGIN_SET);
	ilEnable(IL_FILE_OVERWRITE);
	
	ilGenImages(1,&inpix);
	ilBindImage(inpix);
	
	if(!ilLoadImage(argv[1]))
		return mustard("inpix load fail.",1,1);
	
	if(ilGetInteger(IL_IMAGE_FORMAT)!=IL_COLOUR_INDEX)
		return mustard("inpix not indexed.",1,1);
	
	
	ilConvertPal(IL_PAL_RGB24);
	if(ilGetInteger(IL_PALETTE_NUM_COLS)!=255)
		return mustard("Wrong number of colors.",1,1);
	
	pal=ilGetPalette();
	
	if(strlen(argv[3])<340)
		return mustard("Palette too short.",1,1);
	
	
	for(i=0; i<340; ++i)
		argv[3][i]=b642d(argv[3][i]);
	for(i=0; i<85; ++i)
	{
		pal[3*i  ]=( argv[3][4*i  ]       <<2)|
		           ((argv[3][4*i+1]&0x30) >>4);
		
		pal[3*i+1]=((argv[3][4*i+1]&0x0f) <<4)|
		           ((argv[3][4*i+2]&0x3c) >>2);
		
		pal[3*i+2]=((argv[3][4*i+2]&0x03) <<6)|
		           ( argv[3][4*i+3]          );
		
		pal[3*i+255]=pal[3*i  ];
		pal[3*i+256]=pal[3*i+1];
		pal[3*i+257]=pal[3*i+2];
		
		pal[3*i+510]=pal[3*i  ];
		pal[3*i+511]=pal[3*i+1];
		pal[3*i+512]=pal[3*i+2];
	}
	if(!ilSave(IL_PNG,argv[2]))
		return mustard("outpix save fail",1,1);
	
	return mustard("remapped",1,0);
}

unsigned char b642d(char b64)
{
	if(b64 >='A' && b64 <='Z')
		return b64-'A';
	else if(b64 >= 'a' && b64 <= 'z')
		return b64-'a'+26;
	else if(b64 >= '0' && b64 <= '9')
		return b64-'0'+52;
	else if(b64=='+')
		return 62;
	else if(b64=='-')
		return 63;
	else
		return 0;
}

int mustard(const char *t, int m,int e)
{
	if(!q)
		puts(t);
	switch (m)
	{
	case 1:
		ilDeleteImages(1,&inpix);
	case 0:
	default:
		return e;
	}
}